@@ -24,11 +24,46 @@
 #' the effect type (statistic choice) can be varied.  See \code{\link{anova.lm.rrpp}} for more details.  Please
 #' recognize that the type of SS must be chosen prior to running \code{lm.rrpp} and not when applying \code{\link{anova}}
 #' to the \code{lm.rrpp} fit, as design matrices for the linear model must be created first.  Therefore, SS.type
-#' is an argument for \code{lm.rrpp} and effect.type is an argument for \code{\link{anova.lm.rrpp}}.
+#' is an argument for \code{lm.rrpp} and effect.type is an argument for \code{\link{anova.lm.rrpp}}.  If MANOVA
+#' statistics are preferred, eigenvalues can be added with \code{\link{manova.update}} and statistics summarized with
+#' \code{\link{summary.manova.lm.rrpp}}.  See \code{\link{manova.update}} for examples.
 #' 
 #' The \code{\link{coef.lm.rrpp}} function can be used to test the specific coefficients of an lm.rrpp fit.  The test
 #' statistics are the distances (d), which are also standardized (Z-scores).  The Z-scores might be easier to compare,
 #' as the expected values for random distances can vary among coefficient vectors (Adams and Collyer 2016).
+#' 
+#'  \subsection{ANOVA vs. MANOVA}{ 
+#'  Two SSCP matrices are calculated for each linear model effect, for every random permutation: R (Residuals or Random effects) and
+#'  H, the difference between SSCPs for "full" and "reduced" models. (Full models contain and reduced models lack
+#'  the effect tested; SSCPs are hypothesized to be the same under a null hypothesis, if there is no effect.  The 
+#'  difference, H, would have a trace of 0 if the null hypothesis were true.)  In RRPP, ANOVA and MANOVA correspond to
+#'  two different ways to calculate statistics from R and H matrices.
+#'  
+#'  ANOVA statistics are those that find the trace of R and H SSCP matrices before calculating subsequent statistics,
+#'  including sums of squares (SS), mean squares (MS), and F-values.  These statistics can be calculated with univariate data
+#'  and provide univariate-like statistics for multivariate data.  These statistics are dispersion measures only (covariances
+#'  among variables do not contribute) and are the same as "distance-based" stats proposed by Goodall (1991) and Anderson (2001).
+#'  MANOVA stats require multivariate data and are implicitly affected by variable covariances.  For MANOVA, the inverse of R times
+#'  H (invR.H) is first calculated for each effect, then eigenanalysis is performed on these matrix products.  Multivariate
+#'  statistics are calculated from the positive, real eigenvalues.  In general, inferential
+#'  conclusions will be similar with either approach, but effect sizes might differ.
+#'  
+#'  ANOVA tables are generated by \code{\link{anova.lm.rrpp}} on lm.rrpp fits and MANOVA tables are generated
+#'  by \code{\link{summary.manova.lm.rrpp}}, after running manova.update on lm.rrpp fits.
+#'  
+#'  Currently, mixed model effects are only possible with $ANOVA statistics, not $MANOVA.
+#'  
+#'  More detail is found in the vignette, ANOVA versus MANOVA.
+#' }
+#' 
+#'  \subsection{Notes for RRPP 0.3.1 and subsequent versions}{ 
+#'  F-values via RRPP are calculated with residual SS (RSS) found uniquely for any model terms, as per
+#'  Anderson and ter Braak (2003).  This method uses the random pseudo-data generated by each term's
+#'  null (reduced) model, meaning RSS can vary across terms.  Previous versions used an intercept-only 
+#'  model for generating random pseudo-data.  This generally has appropriate type I error rates but can have
+#'  elevated type I error rates if the observed RSS is small relative to total SS.  Allowing term by term
+#'  unique RSS alleviates this concern.
+#' }
 #'
 #' @param f1 A formula for the linear model (e.g., y~x1+x2).  Can also be a linear model fit
 #' from \code{\link{lm}}.
@@ -73,14 +108,14 @@
 #' is a list of reordered sequences of 1:n, for how residuals were randomized.}
 #' @references Anderson MJ. 2001. A new method for non-parametric multivariate analysis of variance.
 #'    Austral Ecology 26: 32-46.
-#' @references Anderson MJ. and C.J.F. terBraak. 2003. Permutation tests for multi-factorial analysis of variance.
+#' @references Anderson MJ. and C.J.F. ter Braak. 2003. Permutation tests for multi-factorial analysis of variance.
 #'    Journal of Statistical Computation and Simulation 73: 85-113.
 #' @references Collyer, M.L., D.J. Sekora, and D.C. Adams. 2015. A method for analysis of phenotypic change for phenotypes described
 #' by high-dimensional data. Heredity. 115:357-365.
 #' @references Adams, D.C. and M.L. Collyer. 2016.  On the comparison of the strength of morphological integration across morphometric
 #' datasets. Evolution. 70:2623-2631.
 #' @references Adams, D.C and M.L. Collyer. 2018. Multivariate phylogenetic anova: group-clade aggregation, biological 
-#' challenges, and a refined permutation procedure. Evolution. In press.
+#' challenges, and a refined permutation procedure. Evolution. 72:1204-1215.
 #' @seealso \code{procD.lm} and \code{procD.pgls} within \code{geomorph}; \code{\link[stats]{lm}} for more on linear model fits.
 #' @examples
 #' 
@@ -230,40 +265,61 @@
 #' plot(predict(fitGLSm, sizeDF), PC= TRUE) # Independent error
 
 lm.rrpp <- function(f1, iter = 999, seed = NULL, int.first = FALSE,
-                    RRPP = TRUE, SS.type = c("I", "II", "III"),
-                    data = NULL, Cov = NULL,
-                    print.progress = TRUE, Parallel = FALSE, ...) {
+                     RRPP = TRUE, SS.type = c("I", "II", "III"),
+                     data = NULL, Cov = NULL,
+                     print.progress = TRUE, Parallel = FALSE, ...) {
   if(int.first) ko = TRUE else ko = FALSE
-  if(!is.null(data) && inherits(data, "data.frame")) data <- rrpp.data.frame(data)
-  if(!is.null(data)) data <- droplevels.rrpp.data.frame(data)
+  SS.type <- match.arg(SS.type)
+  
   if(print.progress){
     cat("\nPreliminary Model Fit...\n")
     pb <- txtProgressBar(min = 0, max = 4, initial = 0, style=3)
     step <- 1
   }
-  SS.type <- match.arg(SS.type)
-  form <- formula(f1)
-  dep <- eval(form[[2]], data, parent.frame())
-  if(inherits(dep, "dist")) {
-    if(any(dep < 0)) stop("Distances in distance matrix cannot be less than 0")
-    D <- dep
-  } else if((is.matrix(dep) || is.data.frame(dep))
-            && isSymmetric(unname(as.matrix(dep)))) {
-    D <- as.dist(dep)
-  } else D <- NULL
+  
+  dots <- list(...)
+  if(length(dots) > 0) {
+    w <- dots$weights
+    o <- dots$offset
+  } else w <- o <- NULL
+  
+  if(!is.null(Cov)) {
+    Cov.name <- deparse(substitute(Cov))
+    Cov.match <- match(Cov.name, names(data))
+    if(length(Cov.match) > 1) stop("More than one object matches covariance matrix name")
+    if(all(is.na(Cov.match))) Cov <- Cov else Cov <- data[[Cov.match]]
+    if(!is.matrix(Cov)) stop("The covariance matrix must be a matrix.")
+  }
+  
+  if(inherits(f1, "lm")) {
+    lm.args <- lm.args.from.lm(f1)
+    lm.args$keep.order <- ko
+    lm.args$pca <- FALSE
+    lm.args$SS.type <- SS.type
+  }
+  
+  if(inherits(f1, "formula")) {
+    lm.args <- lm.args.from.formula(f1, data = data)
+    if(!is.null(w)) lm.args$weights <- w
+    if(!is.null(o)) lm.args$offset <- o
+    lm.args$keep.order <- ko
+    lm.args$pca <- FALSE
+    lm.args$SS.type <- SS.type
+  }
+  
   if(print.progress) {
     step <- 2
     setTxtProgressBar(pb,step)
   }
-  fit.o <- rrpp.fit(f1, data = data, keep.order = ko, pca = FALSE, SS.type = SS.type,...)
-  if(print.progress) {
-    step <- 3
-    setTxtProgressBar(pb,step)
-  }
+  
+  fit.o <- do.call(rrpp.fit, lm.args)
+  
   dimsY <- dim(as.matrix(fit.o$Y))
   n <- dimsY[1]; p <- p.prime <- dimsY[2]
+  
   if(p > n){
-    fit <- rrpp.fit(f1, data = data, keep.order = ko, pca = TRUE, SS.type = SS.type, ...)
+    lm.args$pca <- TRUE
+    fit <- do.call(rrpp.fit, lm.args)
     Y <- as.matrix(fit$Y)
     dimsY <- dim(Y)
     n <- dimsY[1]; p.prime <- dimsY[2]
@@ -271,6 +327,7 @@ lm.rrpp <- function(f1, iter = 999, seed = NULL, int.first = FALSE,
     fit <- fit.o
     Y <- as.matrix(fit$Y)
   }
+  
   if(print.progress) {
     step <- 3
     setTxtProgressBar(pb,step)
@@ -279,21 +336,18 @@ lm.rrpp <- function(f1, iter = 999, seed = NULL, int.first = FALSE,
   id <- rownames(Y)
   ind <- perm.index(n, iter = iter, seed = seed)
   perms <- iter + 1
+  
   if(print.progress) {
     step <- 4
     setTxtProgressBar(pb,step)
     close(pb)
   }
+  
   SS.args <- list(fit = fit, ind = ind, P = NULL,
                   RRPP = RRPP, print.progress = print.progress)
   beta.args <- SS.args
   beta.args$fit <- fit.o
   if(!is.null(Cov)){
-    Cov.name <- deparse(substitute(Cov))
-    Cov.match <- match(Cov.name, names(data))
-    if(length(Cov.match) > 1) stop("More than one object matches covariance matrix name")
-    if(all(is.na(Cov.match))) Cov <- Cov else Cov <- data[[Cov.match]]
-    if(!is.matrix(Cov)) stop("The covariance matrix must be a matrix.")
     dimsC <- dim(Cov)
     if(!all(dimsC == n))
       stop("Either one or both of the dimensions of the covariance matrix do not match the number of observations.")
@@ -309,9 +363,9 @@ lm.rrpp <- function(f1, iter = 999, seed = NULL, int.first = FALSE,
     if(Parallel) {
       if(.Platform$OS.type == "windows") SS <- do.call(SS.iter, SS.args)
       else SS <- do.call(SS.iterPP, SS.args)
-      } else SS <- do.call(SS.iter, SS.args)
+    } else SS <- do.call(SS.iter, SS.args)
     ANOVA <- anova.parts(fit, SS)
-    LM <- list(coefficients = fit.o$wCoefficients.full[[k]],
+    LM <- list(form = formula(f1), coefficients = fit.o$wCoefficients.full[[k]],
                Y = fit.o$Y,  X = fit.o$X, n = n, p = p, p.prime = p.prime,
                QR = fit.o$QRs.full[[k]],
                fitted = fit.o$fitted.full[[k]],
@@ -324,11 +378,12 @@ lm.rrpp <- function(f1, iter = 999, seed = NULL, int.first = FALSE,
                data = fit.o$data, 
                random.coef = betas$random.coef,
                random.coef.distances = betas$random.coef.distances,
-               ols = TRUE, gls = FALSE)
+               model.sets = fit.o$model.sets, ols = TRUE, gls = FALSE)
     PermInfo <- list(perms = perms,
                      perm.method = ifelse(RRPP==TRUE,"RRPP", "FRPP"), 
                      perm.schedule = ind, perm.seed = seed)
-    out <- list(call = match.call(), LM = LM, ANOVA = ANOVA, PermInfo = PermInfo)
+    out <- list(call = match.call(), 
+                LM = LM, ANOVA = ANOVA, PermInfo = PermInfo)
     if(!is.null(Cov)) {
       out$LM$Cov <- Cov
       out$LM$Pcov <- Pcov
@@ -355,7 +410,7 @@ lm.rrpp <- function(f1, iter = 999, seed = NULL, int.first = FALSE,
     }
     SSY <- SS[1]
     df <- n - fit$wQRs.full[[1]]$rank
-    LM <- list(coefficients=fit.o$wCoefficients.full[[1]],
+    LM <- list(form = formula(f1), coefficients=fit.o$wCoefficients.full[[1]],
                Y=fit.o$Y,  X=fit.o$X, n = n, p = p, p.prime = p.prime,
                QR = fit.o$QRs.full[[1]], fitted = fit.o$fitted.full[[1]],
                residuals = fit.o$residuals.full[[1]],
@@ -368,12 +423,13 @@ lm.rrpp <- function(f1, iter = 999, seed = NULL, int.first = FALSE,
                data = fit.o$data, 
                random.coef = betas$random.coef,
                random.coef.distances = betas$random.coef.distances,
-               ols = TRUE, gls = FALSE)
-    ANOVA <- list(df = df, SS = SS, MS = SS/df)
+               model.sets = fit.o$model.sets, ols = TRUE, gls = FALSE)
+    ANOVA <- list(df = df, RSS.model = SS, MS = SS/df)
     PermInfo <- list(perms = perms,
                      perm.method = ifelse(RRPP==TRUE,"RRPP", "FRPP"), 
                      perm.schedule = ind, perm.seed = seed)
-    out <- list(call = match.call(), LM = LM, ANOVA = ANOVA, PermInfo = PermInfo)
+    out <- list(call = match.call(), 
+                LM = LM, ANOVA = ANOVA, PermInfo = PermInfo)
     if(!is.null(Cov)){
       out$LM$Cov <- Cov
       out$LM$Pcov <- Pcov
@@ -386,7 +442,8 @@ lm.rrpp <- function(f1, iter = 999, seed = NULL, int.first = FALSE,
       out$LM$gls.mean <- colMeans(fit.o$X%*%fit.cov$coefficients)
     }
   }
-  if(!is.null(D)) {
+  if(!is.null(fit.o$D)) {
+    D <- fit.o$D
     qrf <- fit$QRs.full[[k]]
     wqrf <- fit$wQRs.full[[k]]
     D.coef <- qr.coef(qrf, D)
@@ -396,14 +453,13 @@ lm.rrpp <- function(f1, iter = 999, seed = NULL, int.first = FALSE,
   }
   centroid <- gls.centroid <- NULL
   int <- matrix(attr(fit$Terms, "intercept"), n)
-  centroid <- lm.fit(int, Y)$coefficients
+  centroid <- lm.fit(int, fit.o$wY)$coefficients
   out$LM$centroid <- centroid
   if(!is.null(Cov)) {
-    gls.centroid <- lm.fit(crossprod(Pcov, int), crossprod(Pcov, Y))$coefficients
+    gls.centroid <- lm.fit(crossprod(Pcov, int), crossprod(Pcov, fit.o$Y))$coefficients
     out$LM$gls.centroid <- gls.centroid
   }
   
   class(out) = "lm.rrpp"
   out
-}
-
+}
\ No newline at end of file
